@@ -213,18 +213,28 @@ DESeqDataSet <- function(se, design, ignoreRank=FALSE) {
 
     designVarsNumeric <- sapply(designVars, function(v) is.numeric(colData(se)[[v]]))
     if (any(designVarsNumeric)) {
-      warnIntVars <- FALSE
+      msgIntVars <- FALSE
+      msgCenterScale <- FALSE
       for (v in designVars[designVarsNumeric]) {
         if (all(colData(se)[[v]] == round(colData(se)[[v]]))) {
-          warnIntVars <- TRUE
+          msgIntVars <- TRUE
+        }
+        if (mean(colData(se)[[v]]) > 5 | sd(colData(se)[[v]]) > 5) {
+          msgCenterScale <- TRUE
         }
       }
-      if (warnIntVars) {
-        message("  the design formula contains a numeric variable with integer values,
+      if (msgIntVars) {
+        message("  the design formula contains one or more numeric variables with integer values,
   specifying a model with increasing fold change for higher values.
   did you mean for this to be a factor? if so, first convert
   this variable to a factor using the factor() function")
       }
+      if (msgCenterScale) {
+        message("  the design formula contains one or more numeric variables that have mean or
+  standard deviation larger than 5 (an arbitrary threshold to trigger this message).
+  it is generally a good idea to center and scale numeric variables in the design
+  to improve GLM convergence.")
+      }
     }
 
     if (any(designVarsClass == "ordered")) {
@@ -403,7 +413,7 @@ DESeqDataSetFromTximport <- function(txi, colData, design, ...)
 #' @rdname DESeqResults
 #' @export
 setClass("DESeqResults",
-         contains="DataFrame",
+         contains="DFrame",
          representation = representation( 
            priorInfo = "list")
          )
@@ -411,8 +421,8 @@ setClass("DESeqResults",
 #' DESeqResults object and constructor
 #'
 #' This constructor function would not typically be used by "end users".
-#' This simple class extends the DataFrame class of the IRanges package
-#' to allow other packages to write methods for results
+#' This simple class indirectly extends the DataFrame class defined in the
+#' S4Vectors package to allow other packages to write methods for results
 #' objects from the DESeq2 package. It is used by \code{\link{results}}
 #' to wrap up the results table.
 #'
