@@ -4,10 +4,11 @@
 #' results table from \code{DESeq} run without LFC shrinkage.
 #' For consistency with \code{results}, the column name \code{lfcSE}
 #' is used here although what is returned is a posterior SD.
-#' Three shrinkage esimators for LFC are available via \code{type}.
+#' Three shrinkage estimators for LFC are available via \code{type}
+#' (see the vignette for more details on the estimators).
+#' The apeglm publication demonstrates that 'apeglm' and 'ashr' outperform
+#' the original 'normal' shrinkage estimator.
 #'
-#' As of DESeq2 version 1.18, \code{type="apeglm"} and \code{type="ashr"}
-#' are new features, and still under development.
 #' Specifying \code{apeglm} passes along DESeq2 MLE log2
 #' fold changes and standard errors to the \code{apeglm} function
 #' in the apeglm package, and re-estimates posterior LFCs for
@@ -19,14 +20,10 @@
 #' See vignette for a comparison of shrinkage estimators on an example dataset.
 #' For all shrinkage methods, details on the prior is included in
 #' \code{priorInfo(res)}, including the \code{fitted_g} mixture for ashr.
-#' The integration of shrinkage methods from
-#' external packages will likely evolve over time. We will likely incorporate an
-#' \code{lfcThreshold} argument which can be passed to apeglm
-#' to specify regions of the posterior at an arbitrary threshold.
 #'
 #' For \code{normal}, and design as a formula, shrinkage cannot be applied
 #' to coefficients in a model with interaction terms. For \code{normal}
-#' and user-supplied model matrices, shrinkage is only supported via \code{coef}.
+#' and user-supplied model matrices, shrinkage is only supported via \code{coef}. 
 #' For \code{normal} with numeric- or list-style contrasts, it is possible to use \code{lfcShrink},
 #' but likely easier to use \code{DESeq} with \code{betaPrior=TRUE} followed by \code{results},
 #' because the numeric or list should reference the coefficients from the expanded model matrix.
@@ -37,6 +34,7 @@
 #' consult \code{resultsNames(dds)} after running \code{DESeq(dds)}.
 #' note: only \code{coef} or \code{contrast} can be specified, not both.
 #' \code{apeglm} requires use of \code{coef}.
+#' For \code{normal}, one of \code{coef} or \code{contrast} must be provided.
 #' @param contrast see argument description in \code{\link{results}}.
 #' only \code{coef} or \code{contrast} can be specified, not both.
 #' @param res a DESeqResults object. Results table produced by the
@@ -139,20 +137,30 @@ lfcShrink <- function(dds, coef, contrast, res,
   type <- match.arg(type, choices=c("normal","apeglm","ashr"))
   format <- match.arg(format, choices=c("DataFrame", "GRanges","GRangesList"))
   if (length(resultsNames(dds)) == 0) {
-    stop("first run DESeq() before running lfcShrink()")
+    if (type != "apeglm" | (type == "apeglm" & apeAdapt)) {
+      stop("first run DESeq() before running lfcShrink()")
+    }
   }
-  if (attr(dds,"betaPrior")) {
+  betaPrior <- attr(dds,"betaPrior")
+  if (!is.null(betaPrior) && betaPrior) {
     stop("lfcShrink() should be used downstream of DESeq() with betaPrior=FALSE (the default)")
   }
   stopifnot(length(lfcThreshold) == 1 && lfcThreshold >= 0)
+  resultsNamesDDS <- resultsNames(dds)
+  # we can run apeglm without running nbinomWaldTest()
+  # but we need to go get the column names of the model matrix first...
+  if (type == "apeglm" & !apeAdapt & length(resultsNames(dds)) == 0) {
+    resultsNamesDDS <- colnames(model.matrix(design(dds), data=colData(dds)))
+  }
+  # checks on the coef numeric or character wrt resultsNames(dds)
   if (!missing(coef)) {
     if (is.numeric(coef)) {
-      stopifnot(coef <= length(resultsNames(dds)))
-      coefAlpha <- resultsNames(dds)[coef]
+      stopifnot(coef <= length(resultsNamesDDS))
+      coefAlpha <- resultsNamesDDS[coef]
       coefNum <- coef
     } else if (is.character(coef)) {
-      stopifnot(coef %in% resultsNames(dds))
-      coefNum <- which(resultsNames(dds) == coef)
+      stopifnot(coef %in% resultsNamesDDS)
+      coefNum <- which(resultsNamesDDS == coef)
       coefAlpha <- coef
     }
   }
@@ -171,19 +179,26 @@ lfcShrink <- function(dds, coef, contrast, res,
     }
     stopifnot(all(rownames(dds) == rownames(res)))
     if (parallel) {
-      nworkers <- BPPARAM$workers
+      nworkers <- getNworkers(BPPARAM)
       parallelIdx <- factor(sort(rep(seq_len(nworkers),length.out=nrow(dds))))
     }
   }
   
   if (type == "normal") {
 
+    if (missing(coef) & missing(contrast)) {
+      stop("type='normal' requires either 'coef' or 'contrast' be specified.")
+    }
+    
     ############
     ## normal ##
     ############
 
     if (!quiet) message("using 'normal' for LFC shrinkage, the Normal prior from Love et al (2014).
-additional priors are available via the 'type' argument, see ?lfcShrink for details")
+
+Note that type='apeglm' and type='ashr' have shown to have less bias than type='normal'.
+See ?lfcShrink for more details on shrinkage type, and the DESeq2 vignette.
+Reference: https://doi.org/10.1093/bioinformatics/bty895")
     
     if (is(design(dds), "formula")) {
       if (attr(dds, "modelMatrixType") == "user-supplied") {
@@ -299,10 +314,13 @@ additional priors are available via the 'type' argument, see ?lfcShrink for deta
     if (!missing(contrast)) {
       stop("type='apeglm' shrinkage only for use with 'coef'")
     }
-    stopifnot(!missing(coef))    
-    incomingCoef <- gsub(" ","_",sub("log2 fold change \\(MLE\\): ","",mcols(res)[2,2]))
-    if (coefAlpha != incomingCoef) {
-      stop("'coef' should specify same coefficient as in results 'res'")
+    stopifnot(!missing(coef))
+    # if we are using adaptive prior, get the LFC columns
+    if (apeAdapt) {
+      incomingCoef <- gsub(" ","_",sub("log2 fold change \\(MLE\\): ","",mcols(res)[2,2]))
+      if (coefAlpha != incomingCoef) {
+        stop("'coef' should specify same coefficient as in results 'res'")
+      }
     }
 
     if (!quiet) message("using 'apeglm' for LFC shrinkage. If used in published research, please cite:
@@ -311,7 +329,8 @@ additional priors are available via the 'type' argument, see ?lfcShrink for deta
     Bioinformatics. https://doi.org/10.1093/bioinformatics/bty895")
     
     Y <- counts(dds)
-    if (attr(dds, "modelMatrixType") == "user-supplied") {
+    modelMatrixType <- attr(dds, "modelMatrixType")
+    if (!is.null(modelMatrixType) && modelMatrixType == "user-supplied") {
       design <- attr(dds, "modelMatrix")
     } else {
       design <- model.matrix(design(dds), data=colData(dds))
