@@ -727,7 +727,6 @@ setMethod("show", signature(object="DESeqResults"), function(object) {
 #' defaults to FALSE so that the coefficients are given.
 #' @param ... additional arguments
 #'
-#' @docType methods
 #' @name coef
 #' @rdname coef
 #' @aliases coef coef.DESeqDataSet
@@ -740,7 +739,8 @@ setMethod("show", signature(object="DESeqResults"), function(object) {
 #' dds <- DESeq(dds)
 #' coef(dds)[1,]
 #' coef(dds, SE=TRUE)[1,]
-#' 
+#'
+#' @method coef DESeqDataSet
 #' @export
 coef.DESeqDataSet  <- function(object, SE=FALSE, ...) {
   # Temporary hack for backward compatibility with "old" DESeqDataSet
@@ -759,35 +759,6 @@ coef.DESeqDataSet  <- function(object, SE=FALSE, ...) {
   }
 }
 
-#' Summarize DESeq results
-#'
-#' Print a summary of the results from a DESeq analysis.
-#'
-#' @usage
-#' \method{summary}{DESeqResults}(object, alpha, \dots)
-#' 
-#' @param object a \code{\link{DESeqResults}} object
-#' @param alpha the adjusted p-value cutoff. If not set, this
-#' defaults to the \code{alpha} argument which was used in
-#' \code{\link{results}} to set the target FDR for independent
-#' filtering, or if independent filtering was not performed,
-#' to 0.1.
-#' @param ... additional arguments
-#'
-#' @docType methods
-#' @name summary
-#' @rdname summary
-#' @aliases summary summary.DESeqResults
-#' @author Michael Love
-#'  
-#' @examples
-#'
-#' dds <- makeExampleDESeqDataSet(m=4)
-#' dds <- DESeq(dds)
-#' res <- results(dds)
-#' summary(res)
-#'
-#' @export
 summary.DESeqResults <- function(object, alpha, ...) {
   sval <- "svalue" %in% names(object)
   if (sval) {
@@ -848,6 +819,35 @@ summary.DESeqResults <- function(object, alpha, ...) {
   cat("\n")
 }
 
+#' Summarize DESeq results
+#'
+#' Print a summary of the results from a DESeq analysis.
+#'
+#' @param object a \code{\link{DESeqResults}} object
+#' @param alpha the adjusted p-value cutoff. If not set, this
+#' defaults to the \code{alpha} argument which was used in
+#' \code{\link{results}} to set the target FDR for independent
+#' filtering, or if independent filtering was not performed,
+#' to 0.1.
+#' @param ... additional arguments
+#'
+#' @docType methods
+#' @name summary
+#' @rdname summary
+#' @aliases summary summary,DESeqResults-method
+#' @author Michael Love
+#'  
+#' @examples
+#'
+#' dds <- makeExampleDESeqDataSet(m=4)
+#' dds <- DESeq(dds)
+#' res <- results(dds)
+#' summary(res)
+#'
+#' @method summary DESeqResults
+#' @export
+setMethod("summary", signature(object="DESeqResults"), summary.DESeqResults)
+
 #' Accessors for the 'priorInfo' slot of a DESeqResults object.
 #' 
 #' The priorInfo slot contains details about the prior on log fold changes
