@@ -1,7 +1,8 @@
 ## ----setup, echo=FALSE, results="hide"-----------------------------------
-knitr::opts_chunk$set(tidy=FALSE, cache=TRUE,
-                      dev="png",
-                      message=FALSE, error=FALSE, warning=TRUE)
+knitr::opts_chunk$set(tidy = FALSE,
+                      cache = FALSE,
+                      dev = "png",
+                      message = FALSE, error = FALSE, warning = TRUE)
 
 ## ----quickStart, eval=FALSE----------------------------------------------
 #  dds <- DESeqDataSetFromMatrix(countData = cts,
@@ -37,6 +38,21 @@ ddsTxi <- DESeqDataSetFromTximport(txi,
                                    colData = samples,
                                    design = ~ condition)
 
+## ------------------------------------------------------------------------
+coldata <- samples
+coldata$files <- files
+coldata$names <- coldata$run
+
+## ----echo=FALSE----------------------------------------------------------
+library("tximeta")
+se <- tximeta(coldata, skipMeta=TRUE)
+ddsTxi2 <- DESeqDataSet(se, design = ~condition)
+
+## ----eval=FALSE----------------------------------------------------------
+#  library("tximeta")
+#  se <- tximeta(coldata)
+#  ddsTxi <- DESeqDataSet(se, design = ~ condition)
+
 ## ----loadPasilla---------------------------------------------------------
 library("pasilla")
 pasCts <- system.file("extdata",
